`timescale 1ns / 1ps
// IMPORTANT NOTE!!!!!!!! //
// If you receive the error below:
// 	Pl_Uap_Flow1FitterRuleFastFeedbacks: bad index to sec_nodes array //
// Do the following to fix it:
//		Project -> Design Goals & Strategies
//			Change "Timing Goal" to "Timing Performance"


//////////////////////////////////////////////////////////////////////////////////
// 
//	Keyboard Controller
//
//	Signals
//		mclk: Master CLK - 50MHz
//		PS2C: Clk pin from PS2
//		PS2D: Data pin from PS2
//		btn: Reset button, tied to btn0 on Nexys2
//		switch: Enable switch for LCD. SW0 = 0 will enable, SW1 = 1 will disable
//		keys: Debug pins, contains variables for keyval1,keyval2,keyval3
//		RS: LCD pin
//		R_W: Read/Write LCD Pin
//		e: Enable pin
//		JA: Data pins for LCD
//
//	Important Internal Signals
//		mapped_keyval1: Converted ASCII value for key that is pressed from keyboard
//		index: index to LCD character location. 0 = top left, 15 = top right char
//		char: 16 8-bit registers corresponding to each character of the LCD
//					Use the index variable to index into this array
//
//////////////////////////////////////////////////////////////////////////////////
module keyboard_top(
	input mclk,
	input PS2C,
	input PS2D,
	input btn,
	input switch,
	output [23:0] keys,
	//output [7:0] Led,
	output RS,
	output R_W,
	output e,
	inout [7:0] JA
    );
	
	//Parameters to help write to the LCD
	parameter HELLO = 256'h2048656C6C6F2020454E4545323435204C43442020496E697469616C697A6564;
	parameter BLANK = 256'h2020202020202020202020202020202020202020202020202020202020202020;
	parameter SHORT_BLANK = 8'h20;
	
	 wire clk25, clr, key_pressed;
	 wire [7:0] keyval1, keyval2, keyval3;
	 reg [7:0] mapped_keyval1;//, mapped_keyval2, mapped_keyval3;
	 wire [255:0] lcd_input;
	 reg [7:0] char [0:15];
	 reg [3:0] index;
	 
	 assign clr = btn;
	 assign keys = {keyval1, keyval2, keyval3};
	
	//Only sets up top row. Change this if you'd like to have the bottom row too.
	assign lcd_input = (switch) ? HELLO : 
		{char[0], char[1], char[2], char[3], char[4], char[5], char[6], char[7], 
		char[8], char[9], char[10], char[11], char[12], char[13], char[14], char[15], {16{8'h20}}}; 

	//Below contains the mapping of the keyboard controller to ASCII
	//	Only keys on the NUMPAD are active. 
	always @ (keyval1 or keyval3) begin
		case(keyval1)
			'h70: mapped_keyval1 = 8'h30; // 0
			'h69: mapped_keyval1 = 8'h31; // 1
			'h72: mapped_keyval1 = 8'h32; // 2
			'h7A: mapped_keyval1 = 8'h33; // 3
			'h6B: mapped_keyval1 = 8'h34; // 4
			'h73: mapped_keyval1 = 8'h35; // 5
			'h74: mapped_keyval1 = 8'h36; // 6
			'h6C: mapped_keyval1 = 8'h37; // 7
			'h75: mapped_keyval1 = 8'h38; // 8
			'h7D: mapped_keyval1 = 8'h39; // 9
			'hE0:
				case(keyval3)
					'h4A: mapped_keyval1 = 8'hFD; // divide
					'h5A: mapped_keyval1 = 8'h3D; // Enter (=)
					default: mapped_keyval1 = keyval3;
				endcase
			'h7C: mapped_keyval1 = 8'h2A; // *
			'h7B: mapped_keyval1 = 8'h2D; // -
			'h79: mapped_keyval1 = 8'h2B; // +
			default: mapped_keyval1 = 8'h30;
		endcase
	end
	
	always @ (posedge key_pressed or posedge clr) begin
		if(clr == 1) begin
			char[0] <= 8'h20;
			char[1] <= 8'h20;
			char[2] <= 8'h20;
			char[3] <= 8'h20;
			char[4] <= 8'h20;
			char[5] <= 8'h20;
			char[6] <= 8'h20;
			char[7] <= 8'h20;
			char[8] <= 8'h20;
			char[9] <= 8'h20;
			char[10] <= 8'h20;
			char[11] <= 8'h20;
			char[12] <= 8'h20;
			char[13] <= 8'h20;
			char[14] <= 8'h20;
			char[15] <= 8'h20;
			index <= 0;
		end else begin
			char[index] <= mapped_keyval1;
			index <= index + 1;	
			//////////////////////////////////////
			//									//
			// The rest of your edits go here 	//
			//	Note: If you are controlling	//
			//			the value of "index", 	//
			//			you need to comment		//
			//			the above line that		//
			//			increments index		//
			//									//
			//////////////////////////////////////
		end
	end
	 
	 
	 //DONT TOUCH ANYTHING BELOW THIS POINT!
	 clkdiv U1 (.mclk(mclk), .clr(clr), .clk25(clk25));
	 keyboard_ctrl U2 (.clk25(clk25), .clr(clr), .PS2C(PS2C), .PS2D(PS2D), .keyval1(keyval1), .keyval2(keyval2), .keyval3(keyval3), .key_pressed(key_pressed));
	 LCD_DRIVER U3 (.clk(mclk), .reset(clr), .lcd_input(lcd_input), .RS(RS), .R_W(R_W), .e(e), .JA(JA));	


endmodule
